# ../addons/eventscripts/client_keypress/client_keypress.py

import es
import keyfile
import os
import spe

informations = es.AddonInfo()
informations.author = "L'In20Cible"
informations.basename = "client_keypress"
informations.description = "Provides an event fired when a player press/unpress some keys."
informations.name = "Client KeyPress"
informations.url = "http://addons.eventscripts.com/addons/view/client_keypress/"
informations.version = "1.5"

es.ServerVar("client_keypress_version", informations.version, informations.description).makepublic()

players = dict()

def load():
    Initialize()
    es.addons.registerTickListener(TickListenerCallBack)
    es.loadevents("declare", "addons/eventscripts/%s/configuration/events.res" % informations.basename)
    
def Initialize():
    global buttons, offsets
    buttons = dict()
    for button, flag in parseKeyFile(getConfigPath("buttons.cfg")).items():
        if flag.isdigit():
            buttons[button] = int(flag)
    offsets = dict()
    for offset, data in parseKeyFile(getConfigPath("offsets.cfg"))[es.getGameName()].items():
        if data.has_key("offset") and data.has_key("type"):
            offsets[offset] = {
                "offset": int(data["offset"]),
                "type": data["type"]
            }
            if os.name != "nt":
                offsets[offset] += 20
    es.loadevents("addons/eventscripts/%s/configuration/events.res" % informations.basename)
    
def unload():
    es.addons.unregisterTickListener(TickListenerCallBack)
    
def fireEvent(userid, keyname, status):
    es.event("initialize", "client_keypress")
    es.event("setint", "client_keypress", "userid", userid)
    es.event("setstring", "client_keypress", "keyname", keyname)
    es.event("setint", "client_keypress", "status", 1 if status else 0)
    es.event("fire", "client_keypress")
    
def getConfigPath(*subdirectories):
    return os.path.join(es.getAddonPath(informations.basename), "configuration", *subdirectories).replace(os.sep, "/")
    
def getPlayerOffset(userid, offset):
    return_value = None
    if es.exists("userid", userid) and offsets.has_key(offset):
        return_value = spe.getLocVal(offsets[offset]["type"], spe.getPlayer(int(userid)) + offsets[offset]["offset"])
    return return_value
    
def parseKeyFile(file_path):
    return_value = dict()
    if os.path.isfile(file_path):
        try:
            return_value = keyfile.Parse(file_path)
        except keyfile.ParseError:
            pass
    return return_value
    
def TickListenerCallBack():
    for userid in es.getUseridList():
        m_nButtons = getPlayerOffset(userid, "m_nButtons")
        if m_nButtons is not None:
            if not players.has_key(userid):
                players[userid] = m_nButtons
            for button in buttons:
                if m_nButtons & buttons[button] and not players[userid] & buttons[button]:
                    fireEvent(userid, button, True)
                elif players[userid] & buttons[button] and not m_nButtons & buttons[button]:
                    fireEvent(userid, button, False)
            players[userid] = m_nButtons
            
def es_map_start(event_var):
    Initialize()
    
def player_disconnect(event_var):
    userid = int(event_var["userid"])
    if userid in players:
        del players[userid]